#include <stdlib.h>
#include <petscoptions.h>
#include <petsc.h>
#include "nmd.h"
#include "anamod.h"
#include "anamodsalsamodules.h"
#if defined(HAVE_MYSQL)
#include "mysql.h"      
MYSQL *dbase;
#endif

#undef __FUNCT__
#define __FUNCT__ "get_matrix"
static PetscErrorCode get_matrix(Mat *A,PetscTruth *success)
{
  MPI_Comm comm = MPI_COMM_WORLD; char option[1000];
  PetscTruth has; PetscViewer binary_dump; PetscErrorCode ierr;

  PetscFunctionBegin;
  *success = PETSC_FALSE;

  ierr = PetscOptionsGetString
    (PETSC_NULL,"-petsc",option,1000,&has); CHKERRQ(ierr);
  if (has) {
    PetscPrintf(comm,"reading matrix from petsc dump <%s>\n",option);
    ierr = PetscViewerBinaryOpen
      (comm,option,FILE_MODE_READ,&binary_dump); CHKERRQ(ierr);
    ierr = MatLoad(binary_dump,MATSEQAIJ,A); CHKERRQ(ierr);
    ierr = PetscViewerDestroy(binary_dump); CHKERRQ(ierr);
    *success = PETSC_TRUE; goto exit;
  }

  ierr = PetscOptionsGetString
    (PETSC_NULL,"-matrixmarket",option,1000,&has); CHKERRQ(ierr);
  if (has) {
    PetscPrintf(comm,"matrix market read not yet supported\n");
    *success = PETSC_FALSE; goto exit;
  }

 exit:
  CHKMEMQ;
  PetscFunctionReturn(0);
}

#undef __FUNCT__
#define __FUNCT__ "analyze_matrix"
PetscErrorCode analyze_matrix(Mat A,NMD_metadata nmd)
{
  char **categories,**modules;
  AnalysisDataType *types; int icat,imod,ncat,nmod;
  PetscErrorCode ierr;

  PetscFunctionBegin;
  ierr = GetCategories(&categories,&ncat); CHKERRQ(ierr);
  for (icat=0; icat<ncat; icat++) {
    ierr = CategoryGetModules
      (categories[icat],&modules,&types,PETSC_NULL,&nmod); CHKERRQ(ierr);
    for (imod=0; imod<nmod; imod++) {
      AnalysisItem res; int reslen; PetscTruth success;
      ierr = ComputeQuantity
	((AnaModNumericalProblem)A,categories[icat],modules[imod],
	 &res,&reslen,&success); CHKERRQ(ierr);
#if defined(HAVE_MYSQL)
      
#endif
      if (success) {
	AnalysisDataType type;
	ierr = GetDataType
	  (categories[icat],modules[imod],&type); CHKERRQ(ierr);
	ierr = NMDSetArrayValue
	  (nmd,categories[icat],modules[imod],
	   (NMDDataType)type,(void*)&res,reslen); CHKERRQ(ierr);
      }
    }
  }
  PetscFunctionReturn(0);
}


#undef __FUNCT__
#define __FUNCT__ "main"
int main(int argc,char **argv)
{
  Mat A; NMD_metadata nmd; PetscTruth table,success;
  PetscErrorCode ierr;

  PetscFunctionBegin;
  PetscInitialize(&argc,&argv,0,0);
  ierr = AnaModInitialize(); CHKERRQ(ierr);
  ierr = AnaModRegisterSalsaModules(); CHKERRQ(ierr);

  ierr = PetscOptionsHasName(PETSC_NULL,"-table",&table); CHKERRQ(ierr);
  if (table) {
#if defined(HAVE_MYSQL)
    printf("CREATE TABLE anamod_data (\n");
    {
      int ncat,icat,iwrite=0; char **cats;
      ierr = GetCategories(&cats,&ncat); CHKERRQ(ierr);
      for (icat=0; icat<ncat; icat++) {
	int ncmp,icmp; char **cmps; AnalysisDataType *types;
	ierr = CategoryGetModules
	  (cats[icat],&cmps,&types,NULL,&ncmp); CHKERRQ(ierr);
	for (icmp=0; icmp<ncmp; icmp++) {
	  char *type;
	  if (iwrite>0) printf(",\n"); iwrite++;
	  ierr = AnaModGetTypeMySQLName(types[icmp],&type); CHKERRQ(ierr);
	  printf("  `%s-%s` %s",cats[icat],cmps[icmp],type);
	  
	}
      }
    }
/*   dbase = mysql_init(NULL); */
/*   mysql_real_connect(dbase,MYSQLHOST,MYSQLUSER,MYSQLPASS,MYSQLDBASE, */
/* 		     0,NULL,0); */
#else
    SETERRQ(1,"Anamod was compiled without MySQL support");
#endif
    printf("\n)\n");
    goto finalize;
  }
  ierr = get_matrix(&A,&success); CHKERRQ(ierr);
  if (!success) goto exit;

  ierr = NMDCreateObject(&nmd); CHKERRQ(ierr);
  ierr = analyze_matrix(A,nmd); CHKERRQ(ierr);
  ierr = NMDViewObject(nmd); CHKERRQ(ierr);
  {
    char *key,*val;
    ierr = NMDReportObject(nmd,NMDFalse,&key,&val,'\t',0,0); CHKERRQ(ierr);
    printf("Keys: %s\nValues: %s\n",key,val);
    //NMD_FREE(key); NMD_FREE(val);
  }
  ierr = NMDDestroyObject(nmd); CHKERRQ(ierr);

 exit:
#if defined(HAVE_MYSQL)
  mysql_close(dbase);
 finalize:
#endif
  ierr = AnaModDeregisterSalsaModules(); CHKERRQ(ierr);
  ierr = AnaModFinalize(); CHKERRQ(ierr);
  PetscFinalize();
  PetscFunctionReturn(0);
}
